﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------
	
nUI_ButtonBarLayouts =
{
	-- the Bliz "bottom left" bar

	["nUI_MultiBarBottomLeft"] =
	{		
		btn_size = 54,
		gap      = 2,
		anchor   = "BOTTOMRIGHT",
		yOfs     = 55,
		xOfs     = -168,
		rows     = 1,
		cols     = 12,
		page     = 6,
		binding  = "MULTIACTIONBAR1BUTTON",
		label    = "pg6 / BottomLeft",
	},
	
	-- the Bliz "bottom right" bar

	["nUI_MultiBarBottomRight"] =
	{		
		btn_size = 54,
		gap      = 2,
		anchor   = "BOTTOMLEFT",
		yOfs     = 55,
		xOfs     = 168,
		rows     = 1,
		cols     = 12,
		page     = 5,
		binding  = "MULTIACTIONBAR2BUTTON",
		label    = "pg5 / BottomRight",
	},
	
	-- the Bliz "right 1" bar (aka "left" bar in the Bliz code)

	["nUI_MultiBarLeft"] =
	{		
		btn_size = 42.8,
		gap      = 2,
		anchor   = "TOPRIGHT",
		yOfs     = -110,
		xOfs     = -168,
		rows     = 2,
		cols     = 6,
		page     = 3,
		binding  = "MULTIACTIONBAR3BUTTON",
		label    = "pg3 / Right 2",
	},
		
	-- the Bliz "right 2" bar (aka "right" bar in the Bliz code)
	
	["nUI_MultiBarRight"] =
	{		
		btn_size = 42.8,
		gap      = 2,
		anchor   = "TOPLEFT",
		yOfs     = -110,
		xOfs     = 168,
		rows     = 2,
		cols     = 6,
		page     = 4,
		binding  = "MULTIACTIONBAR4BUTTON",
		label    = "pg4 / Right",
	},
	
	-- bonus bar 1

	["nUI_BonusBar1"] =
	{		
		btn_size = 42.8,
		gap      = 2,
		anchor   = "TOPLEFT",
		yOfs     = -110,
		xOfs     = -682,
		rows     = 2,
		cols     = 5,
		base_id  = 108,
		binding  = "NUIBAR1BUTTON",
		label    = "nUI Bar 1",
	},
		
	-- bonus bar 2

	["nUI_BonusBar2"] =
	{		
		btn_size = 42.8,
		gap      = 2,
		anchor   = "TOPRIGHT",
		yOfs     = -110,
		xOfs     = 682,
		rows     = 2,
		cols     = 5,
		base_id  = 118,
		binding  = "NUIBAR2BUTTON",
		label    = "nUI Bar 2",
	},
};
	
nUI_ButtonBars = CreateFrame( "Frame", "nUI_ButtonBars", nUI_Dashboard );
nUI_ButtonBars.Bars = {};

nUI_ButtonBars:RegisterEvent( "VARIABLES_LOADED" );
nUI_ButtonBars:SetScript( "OnEvent", 
	
	function()
		
		local bars = nUI_ButtonBars.Bars;
		
		for i in pairs( nUI_ButtonBars.Bars ) do
			
			nUI_ButtonBars.Bars[i]:UnregisterEvent( "DISPLAY_SIZE_CHANGED" );
			nUI_ButtonBars.Bars[i]:Hide();
			
		end
		
		nUI_ButtonBars.Bars = {};
		
		for i in pairs( nUI_ButtonBarLayouts ) do
			
			nUI_ButtonBars.Bars[i] = nUI_ButtonBars.createBar( i, nUI_ButtonBarLayouts[i] );
					
		end
	end
);

nUI_ButtonBars.createBar = function( name, layout )
	
	local bar  = CreateFrame( "Frame", name, nUI_Dashboard, "SecureStateHeaderTemplate" );
	bar.events = CreateFrame( "Frame", name.."_Events", nUI_ButtonBars );
	bar.layout = layout;
	
	bar:SetBackdrop(
		{
			bgFile   = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBg.blp", 
			edgeFile = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBorder.blp", 
			tile     = true, 
			tileSize = 1, 
			edgeSize = 12, 
			insets   = {left = 0, right = 0, top = 0, bottom = 0},
		}
	);
	
	bar:SetBackdropColor( 0, 0, 0, 0.35 );

	if layout.page then layout.base_id = (layout.page-1) * 12; end
	
	-- button initialization
	
	bar.Buttons = {};

	for i=1,layout.rows do
	
		bar.Buttons[i] = {};
		
		for j=1,layout.cols do
			
			local id         = (i-1)*layout.cols + j;
			local button     = CreateFrame( "CheckButton", "$parent_Button"..id, bar, "ActionBarButtonTemplate" );
			local key1, key2 = GetBindingKey( layout.binding..i );
			
			bar.Buttons[i][j] = button;
		
			button.action                   = layout.base_id + id;
			button.actionType               = layout.binding;
			button.actionID                 = id;
			button.nUI_CachedClearAllPoints = button.ClearAllPoints;
			button.nUI_CachedSetAllPoints   = button.SetAllPoints;
			button.nUI_CachedSetPoint       = button.SetPoint;
		
			button.ClearAllPoints = function() end;
			button.SetAllPoints   = function() end;
			button.SetPoint       = function() end;
	
			button:SetAttribute( "action", button.action ); 
						
		end
	end

	-- manage the layout of the bar
	
	bar.events:RegisterEvent( "DISPLAY_SIZE_CHANGED" );
	bar.events:RegisterEvent( "PLAYER_LOGIN" );
	bar.events:SetScript( "OnEvent", function() bar.events.layoutFrame(); end );
	
	bar.events.layoutFrame = function( layout )
	
		if layout then bar.layout = layout;
		else layout = bar.layout;
		end
		
		nUI:setScale();
					
		if bar.fCache then nUI:lockFrame( bar, false ); end
		
		local btn_size    = layout.btn_size * nUI.scale;
		local btn_gap     = layout.gap * nUI.scale;
		
		bar:SetScale( 1.0 / nUI.scale );
		
		bar:SetFrameStrata( nUI_Dashboard:GetFrameStrata() );
		bar:SetFrameLevel( nUI_Dashboard:GetFrameLevel()+2 );
		
		for row in pairs( bar.Buttons ) do
			
			for col in pairs( bar.Buttons[row] ) do
				
				local button = bar.Buttons[row][col];
		
				button:SetFrameStrata( bar:GetFrameStrata() );
				button:SetFrameLevel( bar:GetFrameLevel()+1 );
				button:SetScale( btn_size / button:GetHeight() );
		
				button:nUI_CachedClearAllPoints();
				
				if col == 1 and row == 1 then 
					button:nUI_CachedSetPoint( "TOPLEFT", bar, "TOPLEFT", btn_size * 0.05, -btn_size * 0.05 );
				elseif col == 1 then
					button:nUI_CachedSetPoint( "TOPLEFT", bar.Buttons[row-1][1], "BOTTOMLEFT", 0, -btn_gap );
				else 
					button:nUI_CachedSetPoint( "TOPLEFT", bar.Buttons[row][col-1], "TOPRIGHT", btn_gap, 0 );
				end
				
				nUI:initActionButton( button );
				
			end
		end		
		
		bar:SetHeight( layout.rows * btn_size + (layout.rows-1) * btn_gap + btn_size * 0.1 );
		bar:SetWidth( layout.cols * btn_size + (layout.cols-1) * btn_gap + btn_size * 0.1 );
		bar:SetPoint( layout.anchor, nUI_Dashboard, "CENTER", layout.xOfs * nUI.scale, layout.yOfs * nUI.scale );
		
		nUI:lockFrame( bar, true, layout.label );
	end
	
	return bar;
	
end
