﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------
	
-------------------------------------------------------------------------------
-- local cache

local CreateFrame  = CreateFrame;
local GetNetStats  = GetNetStats;
local GetFramerate = GetFramerate;

-------------------------------------------------------------------------------

nUI.Class.SysInfoBar = nUI.Class:CreatePrototype();
local prototype      = nUI.Class.SysInfoBar.prototype;

-------------------------------------------------------------------------------

function prototype:init( parent, id, width, height )

	self.parent      = parent;
	self.id          = id;
	self.rid         = 1;
	
	self:layoutFrame( width, height, true );
	
end

-------------------------------------------------------------------------------

function prototype:layoutFrame( width, height, is_init )

	if not is_init then self:unregisterEvents(); end
	
	self.timer       = 0;
	self.width       = width;
	self.height      = height;

	-- if we haven't create this object yet, create its frames
	
	if is_init then
		
		self.frame                 = CreateFrame( "Button", "$parent_SysInfo", self.parent );
		self.frame.latency         = CreateFrame( "Frame", "$parent_Latency", self.frame );
		self.frame.latency.texture = self.frame.latency:CreateTexture();
		self.frame.latency.label   = self.frame.latency:CreateFontString();
		self.frame.fps             = CreateFrame( "Frame", "$parent_FrameRate", self.frame );
		self.frame.fps.texture     = self.frame.fps:CreateTexture();
		self.frame.fps.label       = self.frame.fps:CreateFontString();

		self.frame.latency.texture:SetTexture( "Interface\\AddOns\\nUI\\Bars\\Art\\nUI_SysInfoBar.blp" );
		self.frame.latency.texture:SetTexCoord( 0, 0, 0, 1, 1, 0, 1, 1 );
		
		self.frame.fps.texture:SetTexture( "Interface\\AddOns\\nUI\\Bars\\Art\\nUI_SysInfoBar.blp" );
		self.frame.fps.texture:SetTexCoord( 0, 0, 0, 1, 1, 0, 1, 1 );
		
	end
	
	self.frame:SetWidth( self.width );
	self.frame:SetHeight( self.height );
	self.frame:EnableMouse( true );
	
	self.frame.latency:SetWidth( self.width * 0.3 );
	self.frame.latency:SetHeight( self.height );
	self.frame.latency:SetPoint( "RIGHT", self.frame, "CENTER", -self.width / 64, 0 );
	
	self.frame.latency.texture:SetAllPoints( self.frame.latency );
	
	self.frame.latency.label:SetPoint( "TOPRIGHT", self.frame.latency, "TOPLEFT", -2, 4 );
	self.frame.latency.label:SetFont( "Fonts\\ARIALN.TTF", self.height, "OUTLINE" );
	self.frame.latency.label:SetTextColor( 1, 0.83, 0, 1 );
	self.frame.latency.label:SetJustifyH( "RIGHT" );
	self.frame.latency.label:SetJustifyV( "TOP" );
	
	self.frame.fps:SetWidth( self.width * 0.3 );
	self.frame.fps:SetHeight( self.height );
	self.frame.fps:SetPoint( "LEFT", self.frame, "CENTER", self.width / 64, 0 );
	
	self.frame.fps.texture:SetAllPoints( self.frame.fps );
	
	self.frame.fps.label:SetPoint( "TOPLEFT", self.frame.fps, "TOPRIGHT", 2, 4 );
	self.frame.fps.label:SetFont( "Fonts\\ARIALN.TTF", self.height, "OUTLINE" );
	self.frame.fps.label:SetTextColor( 1, 0.83, 0, 1 );
	self.frame.fps.label:SetJustifyH( "LEFT" );
	self.frame.fps.label:SetJustifyV( "TOP" );

	self:registerEvents();
end

-------------------------------------------------------------------------------

function prototype:registerEvents()
	
	self.frame:SetScript( "OnUpdate", function( who, elapsed ) self:onUpdate( elapsed ); end );
	
	self.frame:SetScript( "OnEnter", 
		function()
			MainMenuBarPerformanceBarFrame.hover = 1;
			MainMenuBarPerformanceBarFrame_OnEnter();
		end
	);
	
	self.frame:SetScript( "OnLeave",
		function()
			MainMenuBarPerformanceBarFrame.hover = nil;
			GameTooltip:Hide();
		end
	);
	
end

-------------------------------------------------------------------------------

function prototype:unregisterEvents()
	
	self.frame:SetScript( "OnUpdate", nil );
	self.frame:SetScript( "OnEnter", nil );
	self.frame:SetScript( "OnLeave", nil );
	
end

-------------------------------------------------------------------------------

function prototype:onUpdate( elapsed )
	
	self.timer = self.timer + elapsed;
	
	if self.timer > 1 and nUI_Console:GetAlpha() > 0 then
		
		self.timer = 0;
		
		-- get current stats
		
		local _,_,latency = GetNetStats();
		local framerate   = GetFramerate();
		local frame, r, g, b;
		
		-- update latency
		
		frame = self.frame.latency;

		r, g = self:colorSelect( latency, 600, false );
		
		frame.texture:SetVertexColor( r, g, 0, 1 );
		frame.label:SetText( ("%0.0fMS"):format( latency ) );
		
		-- update frame rate
		
		frame = self.frame.fps;
		
		r, g = self:colorSelect( framerate, 40, true );
		
		frame.texture:SetVertexColor( r, g, 0, 1 );
		frame.label:SetText( ("%0.0fFPS"):format( framerate ) );
		
	end
end

-------------------------------------------------------------------------------

function prototype:SetPoint( point, relative_to, relative_pt, xOfs, yOfs )
	
	self.frame:SetPoint( point, relative_to, relative_pt, xOfs, yOfs );
	
end

-------------------------------------------------------------------------------

function prototype:colorSelect( value, max, up )
	
	local ratio = min( value, max ) / max;
	local r, r1, r2, g, g1, g2;

	if ratio > 0.5 then
		
		ratio = (ratio - 0.5) * 2;
		if up then
			r1 = 1;
			r2 = 0;
			g1 = 0.83;
			g2 = 1;
		else
			r1 = 1;
			r2 = 1;
			g1 = 0.83;
			g2 = 0;
		end
	else
		ratio = ratio * 2;
		if up then
			r1 = 1;
			r2 = 1;
			g1 = 0;
			g2 = 0.83;
		else
			r1 = 0;
			r2 = 1;
			g1 = 1;
			g2 = 0.83;
		end
	end
	
	r = (r2 - r1) * ratio + r1;
	g = (g2 - g1) * ratio + g1;
	
	return r, g;
end

